library flight_script;

import 'libs/dart_api.dart';
import 'libs/lib.dart';
import 'libs/lib_std.dart';
import 'rpn/lexer.dart';
import 'rpn/parser.dart';
import 'vm/machine.dart';

export 'libs/dart_api.dart' show DartCompoundApi, DartSimpleApi;
export 'libs/lib.dart' show Library, LoadLibrary;
export 'vm/interfaces.dart' show VirtualMachine;

class _Sentinel {
  const _Sentinel._();

  static const _Sentinel _instance = _Sentinel._();

  factory _Sentinel() => _instance;

  @override
  String toString() => 'Stack Base';
}

class Interpreter {
  final MachineImpl machine = MachineImpl();

  Interpreter() {
    machine.loadLibrary(libStd);
  }

  List<dynamic> eval(String input) {
    machine.push(_Sentinel());

    final tokens = Lexer.lex(input).toList();
    final program = Parser.parse(tokens).toList();
    machine.run(program);

    final List<dynamic> retval = [];
    while (true) {
      final next = machine.pop().value;
      if (next == _Sentinel()) {
        return retval;
      }
      retval.insert(0, next);
    }
  }

  void bind(String name, dynamic value) => machine.setVar(name, value);
}
