import 'package:flight_script/vm/commands.dart';
import 'package:flight_script/vm/interfaces.dart';
import 'package:flight_script/vm/values.dart';

/// Make all of the `call(Command)` style calls into methods on the machine
extension DartSimpleApi on VirtualMachine {
  void push(dynamic value) => call(Push(value));

  FlightValue pop() => call(const Pop());

  void store({bool shadows = true, FlightValue? key, FlightValue? value}) =>
      call(Store(shadows: shadows, key: key, value: value));

  void load() => call(const Load());

  void execute() => call(const Execute());
}

/// useful compound methods
extension DartCompoundApi on VirtualMachine {
  T popChecked<T, R extends FlightValue<T>>() {
    final value = pop();
    if (value.runtimeType != R) throw TypeError();
    return value.value;
  }

  bool popBool() => popChecked<bool, BoolValue>();

  double popNumber() => popChecked<double, NumberValue>();

  String popString() => popChecked<String, StringValue>();

  dynamic popObject() => popChecked<dynamic, DartObjectValue>();

  Map<FlightValue, FlightValue> popMap() =>
      popChecked<Map<FlightValue, FlightValue>, MapValue>();

  // sets a named variable in the current frame
  void setVar(String name, dynamic value) {
    push(value);
    push(name);
    store();
  }

  /// run a program (dart or flight) in the current frame
  void run(dynamic program) {
    final closure = box(program);
    if (closure.isNotFunction) {
      throw TypeError();
    }
    (closure as Closure).invoke(this);
  }

  /// function call without constantly pushing/popping
  void fcall(FlightValue fn) {
    push(fn);
    execute();
  }

  /// convert the passed value to its flightvalue
  FlightValue box(dynamic value) {
    push(value);
    return pop();
  }
}
