import '../vm/interfaces.dart';
import 'dart_api.dart';
import 'lib.dart';

/// The standard library
/// It is possible to load the vm without this, but it's nearly useless
final libStd = Library({
  "true": true,
  "false": false,
  "dup": (VirtualMachine state) => state.dup(),
  "!": (VirtualMachine state) => state.push(!state.popBool()),
  "+": (VirtualMachine state) => state.push(
        state.popNumber() + state.popNumber(),
      ),
  "-": (VirtualMachine state) =>
      state.push(-state.popNumber() + state.popNumber()),
  "*": (VirtualMachine state) =>
      state.push(state.popNumber() * state.popNumber()),
  "/": (VirtualMachine state) =>
      state.push((1.0 / state.popNumber()) * state.popNumber()),
  "=": (VirtualMachine state) => state.push(state.pop() == state.pop()),
  "!=": (VirtualMachine state) => state.push(state.pop() != state.pop()),
  ">": (VirtualMachine state) =>
      state.push(state.popNumber() < state.popNumber()),
  "<": (VirtualMachine state) =>
      state.push(state.popNumber() > state.popNumber()),
  ">=": (VirtualMachine state) =>
      state.push(state.popNumber() <= state.popNumber()),
  "<=": (VirtualMachine state) =>
      state.push(state.popNumber() >= state.popNumber()),
  "[]*": (VirtualMachine state) => state.push(<FlightValue, FlightValue>{}),
  "[]=": (VirtualMachine state) => state.mapSet(),
  "[]": (VirtualMachine state) => state.mapGet(),
  "[]!": (VirtualMachine state) => state.mapDelete(),
  "if": (VirtualMachine state) => state.if_(),
  "ifelse": (VirtualMachine state) => state.ifelse(),
  "loop": (VirtualMachine state) => state.loop(),
  "print": (VirtualMachine state) => print(state.pop().value),
  "inspect": (VirtualMachine state) => print(state.pop()),
});

extension LibStd on VirtualMachine {
  void dup() {
    final top = pop();
    push(top);
    push(top);
  }

  void if_() {
    final fn = pop();
    if (popBool()) {
      fcall(fn);
    }
  }

  void ifelse() {
    final elseFn = pop();
    final fn = pop();
    popBool() ? fcall(fn) : fcall(elseFn);
  }

  void loop() {
    final end = popNumber();
    final start = popNumber();
    final block = pop();
    for (var i = start; i <= end; i++) {
      push(i.toDouble());
      fcall(block);
    }
  }

  void mapSet() {
    final value = pop();
    final key = pop();
    final map = popMap();
    map[key] = value;
    push(map);
  }

  void mapGet() {
    final key = pop();
    final map = popMap();
    final value = map[key];
    push(map);

    if (value == null) {
      print("here");
    } else {
      push(value);
    }
  }

  void mapDelete() {
    final key = pop();
    final map = pop().value;
    map.remove(key);
    push(map);
  }
}
