import '../vm/commands.dart';
import '../vm/interfaces.dart';
import '../vm/values.dart';
import 'token.dart';

abstract class Parser {
  static Iterable<Command> parse(Iterable<Token> tokens) =>
      _parse(tokens.iterator);

  static Iterable<Command> _parse(Iterator<Token> iterator) sync* {
    while (iterator.moveNext()) {
      final token = iterator.current;
      switch (token.type) {
        case TokenType.number:
          yield Push(double.parse(token.value));
          break;
        case TokenType.symbol:
          yield Push(token.value);
          yield const Load();
          break;
        case TokenType.assignment:
          final shadows = iterator.current.value != "::";
          if (!iterator.moveNext() ||
              iterator.current.type != TokenType.symbol) {
            throw StateError("Assignment not followed by valid symbol");
          }
          yield Store(
              key: StringValue(iterator.current.value), shadows: shadows);
          break;
        case TokenType.execute:
          yield const Execute();
          break;
        case TokenType.functionStart:
          // We must call toList here, or iteration will continue in the wrong order
          yield Push(_parse(iterator).toList(growable: false));
          break;
        case TokenType.functionEnd:
          return;
        case TokenType.String:
          yield Push(token.value);
          break;
      }
    }
  }
}
